--  Part of product Numeric-Name-IO, written in Ada by jpwoodruff@gmail.com,

--  Demonstrate the ability of Name_IO to obtain each of its input types.
with Ada.Numerics.Generic_Real_Arrays;

with Numeric_Io ;
with Numeric_Io.Matrix_Io ;

with Name_Io;
with Name_Io.Matrix_name_IO ;

with Ada.Text_Io;            use Ada.Text_Io ;
with Ada.Integer_text_Io ;   use Ada.Integer_text_Io ;

with Input_Collection;


procedure Test_Name_Io is
   type Real is digits 6 ;
   package Real_Io is new Ada.Text_Io.Float_Io (Real) ;
   use Real_Io ;

   package Scalar_IO is new Numeric_IO (Real, Integer ) ;
   use Scalar_Io ;

   package Ada_Matrix is new Ada.Numerics.Generic_Real_Arrays (Real => Real) ;

   package Matrix_IO is new Scalar_IO.Matrix_IO
     (Vector_Ix    => Integer,
      Vector_Type  => Ada_Matrix.Real_Vector,
      Matrix_Ix1   => Integer,
      Matrix_Ix2   => Integer,
      Matrix_Type  => Ada_Matrix.Real_Matrix) ;

   use Matrix_Io ;

   package Name_List is
      type Input_Names_Type is (Name, Str, I, A, V,
                                Mat, Greek_Letter, Prompted, Default);
   end Name_List;

   type Greek is (Alpha, Beta, Gamma, Epsilon, Zeta);
   Greek_Letter : Greek;
   Name : String (1 .. 10);
   Str  : String (1 .. 40);
   Last : Natural;
   I    : Integer;
   A    : real;
   V    : Ada_Matrix.Real_Vector (1 .. 5);
   Mat  : Ada_Matrix.Real_Matrix (1 .. 3, 1 .. 3);
   Prompted, Default : Integer;

   package Nio is new Name_Io (Names_Type    => Name_List.Input_Names_Type,
                               Floating_Type => Real,
                               Integer_Type  => Integer,
                               Scalar_Io     => Scalar_IO);


   procedure Get_Greek is new Nio.Get_Item (Greek);
   package Greek_Io is new Ada.Text_Io.Enumeration_Io (Greek);
   use Greek_Io;

   package Name_Matrix_Input is new Nio.Matrix_name_IO
     (Matrix_Io  => Matrix_IO) ;

begin
   --
   --  precondition: this program (might have) been invoked
   --  with a file name on its command line.

   Input_Collection.Read_Collection_Files ;

   --  The file contains a collection of input data for
   --  one instance the object class called "testnames".
   --
   --  this test is provided with an accompanying test data file
   --
   Nio.Acquire_Input ("testnames");

   -- object oriented input; interaction will cite the name of the instance
   begin
      Nio.Get (Name_List.Name, Name, Last);
      Nio.Set_Object_Instance_Name (Name (1 .. Last));
   exception
      when Nio.No_Input =>
         Nio.Set_Object_Instance_Name ("Anonymous");
   end;

   Nio.Get (Name_List.I, I);
   Nio.Get (Name_List.A, A);
   Name_Matrix_Input.Get (Name_List.V, V);
   Name_Matrix_Input.Get (Name_List.Mat, Mat);
   Nio.Get (Name_List.Str, Str, Last);
   Get_Greek (Name_List.Greek_Letter, Greek_Letter);

   Nio.Get
     (Name_List.Prompted, Prompted,
      Prompt =>
        "The data input did not include a value for (integer) Prompted");

   begin
      Nio.Get (Name_List.Default, Default);
   exception
      when Nio.No_Input =>
         Put_Line
           ("No value of the input ""Default"" is given, AND no interaction is specified.");
         Put_Line
           ("the exception handler must provide for correct action.");
         Put_Line ("In the example, ""Default"" is given the value 10") ;
         Default := 10;
   end;
   --
   -- now clear the instance data from NIO state,
   -- in preparation for subsequent instances.
   Nio.Forget_Contents;

   New_Line;
   Put_Line ("Now the test program echoes the data that it has acquired:");
   New_Line;
   Put_Line (Str (1 .. Last));
   Put (" I = ");
   Put (I);
   New_Line;
   Put (" A (float) = ");
   Put (A);
   New_Line;
   Put (" V = ");
   Put (V, Aft => Aft (V));
   New_Line;
   Put (" Mat = ");
   New_Line;
   Put (Mat, Aft => Aft (Mat));
   Put (" Greek_Letter = ");
   Put (Greek_Letter);
   New_Line;
   Put (" Data that were prompted for: ");
   Put (Prompted);
   New_Line;
   Put (" Data that took default value: ");
   Put (Default);
   New_Line;

end Test_Name_Io;


-------------------------------------------------------------------------------
--  This is free software; you can redistribute it and/or modify it
--  under terms of the GNU General Public License as published by the
--  Free Software Foundation; either version 2, or (at your option)
--  any later version.  This software is distributed in the hope that
--  it will be useful, but WITH OUT ANY WARRANTY; without even the
--  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
--  PURPOSE. See the GNU General Public License for more details. Free
--  Software Foundation, 59 Temple Place - Suite 330, Boston, MA
--  02111-1307, USA.
--
--  As a special exception, if other files instantiate generics from
--  this unit, or you link this unit with other files to produce an
--  executable, this unit does not by itself cause the resulting
--  executable to be covered by the GNU General Public License. This
--  exception does not however invalidate any other reasons why the
--  executable file might be covered by the GNU Public License.

--  This software was prepared as an account of work sponsored by an
--  agency of the United States Government.  Neither the United States
--  Government nor the University of California nor any of their
--  employees makes any warranty, express or implied, or assumes any
--  liability or responsibility for the accuracy, completeness, or
--  usefulness of any information disclosed.
-------------------------------------------------------------------------------

