--  Part of product Numeric-Name-IO, written in Ada by jpwoodruff@gmail.com,

--  Test Get and Put operations on vectors and matrices using direct
--  interaction.

with Ada.Numerics.Generic_Real_Arrays;

with Ada.Text_Io ;
with Numeric_Io.Matrix_Io ;
procedure Test_Num_Io  is
   type Real is digits 6 ;

   -----------------------------------------------------------
   --  Instantiation of IO services for Ada standard matrix --
   -----------------------------------------------------------
   package Ada_Matrix is new Ada.Numerics.Generic_Real_Arrays (Real => Real) ;

   package Scalar_Io is new Numeric_Io (Real, Integer) ;

   package Matrix_Io is new Scalar_Io.Matrix_Io
     (Vector_Ix    => Integer,
      Vector_Type  => Ada_Matrix.Real_Vector,
      Matrix_Ix1   => Integer,
      Matrix_Ix2   => Integer,
      Matrix_Type  => Ada_Matrix.Real_Matrix) ;
   use Matrix_Io ;


   V  : Ada_Matrix.Real_Vector (1..4) ;
   M  : Ada_Matrix.Real_Matrix (1..3, 1..3) ;

   ------------------------------------------------------------------
   -- The IO packages can also serve arrays indexed by enumeration --
   ------------------------------------------------------------------
   type Letter is (A, B, C) ;
   type Letter_Vector is array (Letter range <>) of Real ;
   type Letter_Matrix is array (Letter range <>, Letter range <>) of Real ;

   package Letter_Matrix_Io is new Scalar_Io.Matrix_Io
     (Vector_Ix   => Letter,
      Vector_Type => Letter_Vector,
      Matrix_Ix1  => Letter,
      Matrix_Ix2  => Letter,
      Matrix_Type => Letter_Matrix) ;

   M2        : Letter_Matrix (Letter, Letter) ;
   Selection : Character ;

begin
   loop
      Scalar_IO.Menu ("Select [V]ector, [M]atrix, [L]etter-indexed matrix, or [Q]uit",
                      Choice => "VMLQ", Item => Selection) ;
      exit when Selection = 'Q' ;
      case Selection is
         when 'V' =>
            Matrix_Io.Get ("Enter a 4-vector", V) ;
            Matrix_Io.Put (V, Aft => Matrix_IO.Aft (V));
            Ada.Text_Io.New_Line ;
         when 'M' =>
            Matrix_Io.Get ("Enter a 3x3 matrix", M) ;
            Matrix_Io.Put (M, Aft => Matrix_Io.Aft(M) ) ;
            Ada.Text_Io.New_Line ;
         when 'L' =>
            Letter_Matrix_Io.Get ("Enter a matrix with indices a..c, a..c", M2) ;
            Letter_Matrix_Io.Put (M2, Aft => Letter_Matrix_Io.Aft(M2)) ;
            Ada.Text_Io.New_Line ;
         when others => null ;   -- will not occur from Menu
      end case ;
   end loop ;
end Test_Num_Io  ;

-------------------------------------------------------------------------------
--  This is free software; you can redistribute it and/or modify it
--  under terms of the GNU General Public License as published by the
--  Free Software Foundation; either version 2, or (at your option)
--  any later version.  This software is distributed in the hope that
--  it will be useful, but WITH OUT ANY WARRANTY; without even the
--  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
--  PURPOSE. See the GNU General Public License for more details. Free
--  Software Foundation, 59 Temple Place - Suite 330, Boston, MA
--  02111-1307, USA.
--
--  As a special exception, if other files instantiate generics from
--  this unit, or you link this unit with other files to produce an
--  executable, this unit does not by itself cause the resulting
--  executable to be covered by the GNU General Public License. This
--  exception does not however invalidate any other reasons why the
--  executable file might be covered by the GNU Public License.

--  This software was prepared as an account of work sponsored by an
--  agency of the United States Government.  Neither the United States
--  Government nor the University of California nor any of their
--  employees makes any warranty, express or implied, or assumes any
--  liability or responsibility for the accuracy, completeness, or
--  usefulness of any information disclosed.
-------------------------------------------------------------------------------

